@@ -1,17 +1,18 @@
 package SPOPS::DBI;
 
-# $Id: DBI.pm,v 1.6 2001/02/25 18:50:19 lachoy Exp $
+# $Id: DBI.pm,v 1.28 2001/06/09 18:06:09 lachoy Exp $
 
 use strict;
-use SPOPS         qw( _wm _w DEBUG );
-use SPOPS::SQLInterface;
-use SPOPS::Secure qw( :level );
-use Carp          qw( carp );
-use DBI           ();
 use Data::Dumper  qw( Dumper );
+use DBI           ();
+use SPOPS         qw( _wm _w DEBUG );
+use SPOPS::Secure qw( :level );
+use SPOPS::SQLInterface;
+use SPOPS::Tie    qw( $PREFIX_INTERNAL );
 
 @SPOPS::DBI::ISA       = qw( SPOPS  SPOPS::SQLInterface );
-@SPOPS::DBI::VERSION   = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
+$SPOPS::DBI::VERSION   = '1.7';
+$SPOPS::DBI::Revision  = substr(q$Revision: 1.28 $, 10);
 
 $SPOPS::DBI::GUESS_ID_FIELD_TYPE = DBI::SQL_INTEGER();
 
@@ -78,15 +79,18 @@ sub class_initialize {
   return 1;
 }
 
+
 # Dummy method for subclasses to override
 
 sub _class_initialize { return 1; }
 
+
 # Override the default SPOPS initialize call so we can use ->{id} and
 # mixed-case fields
 
 sub initialize {
   my ( $self, $p ) = @_;
+  $self->set_all_loaded();
   return unless ( ref $p and scalar( keys %{ $p } ) );
 
   # We allow the user to substitute id => value instead for the
@@ -105,8 +109,9 @@ sub initialize {
   return $self;
 }
 
+
 # Typical call:
-# $self->check_action_security( { required => SEC_LEVEL_WRITE } );
+# $self->check_action_security({ required => SEC_LEVEL_WRITE });
 
 # Note that we return SEC_LEVEL_WRITE to all requests where the object
 # does not have an ID -- meaning that the object has not yet been
@@ -122,13 +127,18 @@ sub check_action_security {
   # might not be true in all cases, provide an escape route for classes
   # that need security and want to handle their ids themselves
 
-  return SEC_LEVEL_WRITE if ( $p->{is_add} ); 
+  return SEC_LEVEL_WRITE if ( $p->{is_add} ); #or ! $self->saved ); 
 
   # If the class has told us they're not using security, then 
   # everyone can do everything
   
   return SEC_LEVEL_WRITE if ( $self->no_security );
 
+  # This gets filled with the found security level, oddly, the user
+  # can pass in a security level if it's already been found
+
+  my $level = $p->{security_level};
+
   # Check to see that the ID exists -- if not, it's an add and will
   # not be checked since SPOPS relies on your application to implement
   # who should and should not create an object. 
@@ -139,41 +149,44 @@ sub check_action_security {
   #
   # http://lists.sourceforge.net/lists/listinfo/openinteract-dev
 
-  my $class = ref $self || $self;
-  my $id    = ( ref $self ) ? $self->id : $p->{id};
-  return SEC_LEVEL_WRITE unless ( $id ); 
-  _w( 1, "Checking action on $class ($id) and required level is ($p->{required})" );
+  my ( $class, $id );
+  unless ( $level ) {
+    $class = ref $self || $self;
+    $id    = ( ref $self ) ? $self->id : $p->{id};
+    return SEC_LEVEL_WRITE unless ( $id ); 
+    DEBUG() && _w( 1, "Checking action on $class ($id) and required",
+                    "level is ($p->{required})" );
 
-  # Calls to SPOPS::Secure->... note that we do not need to explicitly
-  # pass in group/user information, since SPOPS::Secure will retrieve
-  # it for us.
+    # Calls to SPOPS::Secure->... note that we do not need to
+    # explicitly pass in group/user information, since SPOPS::Secure
+    # will retrieve it for us.
 
-  my $level = eval { $class->check_security({ 
-                                class => $class, oid => $id }) };
-  if ( $@ ) {
-    $SPOPS::Error::user_msg = "Cannot check security on for $class ($id)";
-    die $SPOPS::Error::user_msg;
+    $level = eval { $class->check_security({ class     => $class, 
+                                             object_id => $id }) };
+    if ( $@ ) {
+      $SPOPS::Error::user_msg = "Cannot check security for object: $class ($id)";
+      die $SPOPS::Error::user_msg;
+    }
   }
-  
-  _w( 1, "Found security level of ($level)" );
+  DEBUG() && _w( 1, "Found security level of ($level)" );
 
   # If the level is below what is necessary, set an error message and
   # die with a general one.
 
   if ( $level < $p->{required} ) {
-    _w( 1, "Cannot access $class record with ID $id; ", 
-           "access: $level while $p->{required} is required." );
+    DEBUG() && _w( 1, "Cannot access $class record with ID $id; ", 
+                    "access: $level while $p->{required} is required." );
     my $msg = "Action prohibited due to security. Insufficient access for requested action";
-    SPOPS::Error->set( { user_msg => $msg, type => 'security',
-                         system_msg => "Required access: $p->{required}; retrieved access: $level",
-                         extra => { security => $level } } );
+    SPOPS::Error->set({ user_msg   => $msg, 
+                        type       => 'security',
+                        system_msg => "Required access: $p->{required}; retrieved access: $level",
+                        extra      => { security => $level } });
     die $msg;
   }
-
-  # security checks out, we're golden
-  return $level;                
+  return $level; # Rock and roll
 }
 
+
 # Return a snippet suitable for a where clause: page_id = 5 or
 # comment_id = '818172723'
 
@@ -191,22 +204,24 @@ sub id_clause {
   my $db    = $p->{db} || $item->global_db_handle;
   unless ( $db ) {
     my $msg = 'Cannot create ID clause';
-    SPOPS::Error->set( { user_msg => $msg, 
-                         system_msg => 'No db handle available when id_clause routine entered',
-                         method => 'id_clause', type => 'db' } );
+    SPOPS::Error->set({ user_msg   => $msg, 
+                        system_msg => 'No db handle available when id_clause routine entered',
+                        method     => 'id_clause', 
+                        type       => 'db' } );
     die $msg;
   }
   my $id_field  = $item->id_field;
-  my $type_info = eval { $item->db_discover_types( $item->base_table, { dbi_type_info => $p->{dbi_type_info},
-                                                                       db => $db } ); };
+  my $type_info = eval { $item->db_discover_types( $item->base_table, 
+                                                   { dbi_type_info => $p->{dbi_type_info},
+                                                     db            => $db } ) };
 
  # If we cannot get the type via our own system, just guess that the
  # ID field is a number
 
   if ( $@ ) {
     $type_info->{ $id_field } = $SPOPS::DBI::GUESS_ID_FIELD_TYPE;
-   _wm( 0, "Likely was not passed sufficient information to ",
-           "get infromation requested. Making a 'best guess'" );
+   _w( 0, "Likely was not passed sufficient information to ",
+          "get infromation requested. Making a 'best guess'" );
   }
   my $use_id_field = ( $opt eq 'noqualify' ) 
                         ? $id_field 
@@ -214,19 +229,23 @@ sub id_clause {
   return join(' = ', $use_id_field, $db->quote( $id, $type_info->{ $id_field } ) );
 }
 
+
 # Allows the user to define how fields will be formatted
 # in a SELECT (date formatting, substrings, etc)
 
 sub format_select {
   my ( $class, $fields, $conf ) = @_;
+  $conf ||= {};
+  die "Fields passed in not an arrayref: ($fields)" unless ( ref $fields eq 'ARRAY' );
   my @return_fields;
-  my $altered = $class->field_alter;
+  my $altered = $class->field_alter();
   foreach my $field ( @{ $fields } ) {
     push @return_fields, $conf->{ $field } || $altered->{ $field } || $field;
   }
   return \@return_fields;
 }
 
+
 sub get_cached_object {
   my ( $class, $p ) = @_;
   return undef unless ( $p->{id} );
@@ -236,13 +255,14 @@ sub get_cached_object {
   # and assign the values from the cache to it.
 
   if ( my $item_data = $class->global_cache->get({ class => $class, id => $p->{id} }) ) {
-    _w( 1, "Retrieving from cache..." );
+    DEBUG() && _w( 1, "Retrieving from cache..." );
     return $class->new( $item_data );
   }
-  _w( 1, "Cached data not found." );
+  DEBUG() && _w( 1, "Cached data not found." );
   return undef;
 }
 
+
 sub set_cached_object {
   my ( $self, $p ) = @_;
   return undef unless ( ref $self );
@@ -251,7 +271,9 @@ sub set_cached_object {
   return $self->global_cache->set({ data => $self });
 }
 
+
 # Return 1 if we're using the cache; undef if not
+
 sub use_cache {
   my ( $class, $p ) = @_;
   return undef if ( $p->{skip_cache} );
@@ -262,30 +284,27 @@ sub use_cache {
   return 1;
 }
 
+
 sub fetch {
   my ( $class, $id, $p ) = @_;
   $id ||= 0;
   $p  ||= {};
-  _wm( 2, DEBUG_FETCH,
-       "Trying to fetch an item of $class with ID $id and params ", 
-       join " // ",
-            map { $_ . ' -> ' . ( defined( $p->{$_} ) ? $p->{$_} : '' ) }
-                keys %{ $p }
-  );
+  DEBUG() && _wm( 2, DEBUG_FETCH,
+                "Trying to fetch an item of $class with ID $id and params ", 
+                join " // ",
+                map { $_ . ' -> ' . ( defined( $p->{$_} ) ? $p->{$_} : '' ) }
+                    keys %{ $p } );
  
- # Return nothing if we are not passed an ID or if the 
- # ID passed is a temporary one.
-  
+  # No ID, no object
+
   return undef   unless ( $id and $id !~ /^tmp/ );
   
-  # Do *not* wrap this in an eval {}, since we want the 
-  # die to propogate up to the caller if the security
-  # does not check out; if the procedure does not die, 
-  # the security is ok.
-  
-  my ( $level );
+  # Security violations bubble up to caller  
+
+  my $level = $p->{security_level};
   unless ( $p->{skip_security} ) {
-    $level = $class->check_action_security( { id => $id, required => SEC_LEVEL_READ } );
+    $level ||= $class->check_action_security({ id       => $id, 
+                                               required => SEC_LEVEL_READ });
   }
   
   # Do any actions the class wants before fetching -- note that if any
@@ -306,27 +325,17 @@ sub fetch {
     $p->{skip_cache}++;         # Set so we don't re-cache it later
   }
   
-  unless ( $obj ) {
-    
-    # Get the basic fields; note that we use this arrayref below
-    # and it's not just a temporary variable for the next statement :)
-    
-    my $fields = $class->field_list;
-    
-    # Format the fields as the class wants; note that fields
-    # passed into the method (in $p->{field_alter}) override 
-    # anything in the configuration
-    
-    my $field_select = $class->format_select( $fields, $p->{field_alter} );
-    warn " (DBI/fetch): SELECTing: ", join( "//", @{ $field_select } ), "\n"  if ( DEBUG_FETCH );
+  unless ( ref $obj eq $class ) {    
+    my ( $raw_fields, $select_fields ) = $class->_fetch_select_fields( $p );    
+    DEBUG() && _wm( 1, DEBUG_FETCH, "SELECTing: ", join( "//", @{ $select_fields } ) );
     
     # Put all the arguments into a hash (so we can reuse them simply 
-    # later) and Select the record
+    # later) and grab the record
     
-    my %args = ( from => [ $class->table_name ],
-                 select => $field_select,
-                 where => $class->id_clause( $id, undef, $p ),
-                 db => $p->{db},
+    my %args = ( from   => [ $class->table_name ],
+                 select => $select_fields,
+                 where  => $class->id_clause( $id, undef, $p ),
+                 db     => $p->{db},
                  return => 'single' );
     my $row = eval { $class->db_select( \%args ); };
     
@@ -334,13 +343,13 @@ sub fetch {
     
     if ( $@ ) {
       $class->fail_fetch( \%args );   
-      $SPOPS::Error::user_msg = 'Error retrieving record from database';
+      _w( 0, "FETCH failed. Error returned: $@" );
       die $SPOPS::Error::user_msg;
     }
-    
-    # If the row isn't found, return nothing; just as if an incorrect (or nonexistent)
-    # ID were passed in 
-    
+
+    # If the row isn't found, return nothing; just as if an incorrect
+    # (or nonexistent) ID were passed in
+
     return undef unless ( $row );
     
     # Note that we pass $p along to the ->new() method, in case
@@ -350,99 +359,232 @@ sub fetch {
     # of an object -- THAT would be fun to debug...
 
     $obj = $class->new( { id => $id, %{ $p } } );
+    $obj->_fetch_assign_row( $raw_fields, $row );
+  }
+  return $obj->_fetch_post_process( $p, $level );
+}
 
-    # Go through each of the fields and set information.
+sub fetch_group {
+  my ( $class, $p ) = @_;
+  my $table_name    = $class->CONFIG->{base_table};
 
-   _wm( 1, DEBUG_FETCH, "Setting the following fields from row:" );
-    for ( my $i = 0; $i < scalar @{ $row }; $i++ ) {
-      _wm( 1, DEBUG_FETCH, sprintf( " --%-18s: %s", $fields->[ $i ], $row->[ $i ]) );
-      $obj->{ $fields->[ $i ] } = $row->[ $i ];
+  # Take the fields that are just columns and prepend them with the
+  # table name; other columns are assumed to be taken care of
+
+  my ( $raw_fields, $select_fields )  = $class->_fetch_select_fields( $p );
+  for ( my $i = 0; $i < scalar @{ $raw_fields }; $i++ ) {
+    push @{ $p->{select} }, ( $raw_fields->[ $i ] eq $select_fields->[ $i ] )
+                              ? join( '.', $table_name, $raw_fields->[ $i ] )
+                              : $select_fields->[ $i ];
+  }
+  my $sth = $class->_execute_multiple_record_query( $p );
+
+  # initialize limit tracking vars -- the limit passed in can be:
+  # limit => 'n,n'  --> limit => 'offset,max'
+  # limit => 'n'    --> limit => 'max'
+
+  my ($count, $offset, $max) = 0;
+  if ( $p->{limit} ) {
+    if ( $p->{limit} =~ /,/ ) {
+      ( $offset, $max ) = split /\s*,\s*/, $p->{limit};
+      $max += $offset;
+    } 
+    else {
+      $max = $p->{limit};            
+    }
+    DEBUG() && _wm( 1, DEBUG_FETCH, "Limit set: Start $offset to $max" );
+  }
+
+  my @obj_list = ();
+
+ROW:
+  while ( my $row = $sth->fetchrow_arrayref ) {
+    my $obj = $class->new;
+    $obj->_fetch_assign_row( $raw_fields, $row );
+    next ROW unless ( $obj );
+
+    # Check security on the row unless overridden by
+    # 'skip_security'. If the security check fails that's ok, just
+    # skip the row and move on
+
+    my $sec_level = SEC_LEVEL_WRITE;
+    unless ( $p->{skip_security} ) {
+      $sec_level = eval { $obj->check_action_security({
+                                          required => SEC_LEVEL_READ }) };
+      if ( $@ ) {
+        DEBUG() && _wm( 1, DEBUG_FETCH, "Security check for object in ",
+                                        "fetch_group() failed, skipping." );
+        next ROW;
+      }
+    }
+
+    # Not to the offset yet, so go to the next row but still increment
+    # the counter so we calculate limits properly
+    
+    if ( $offset and ( $count < $offset ) ) {
+      $count++;
+      next ROW;
+    }
+    last ROW if ( $max and ( $count >= $max ) );
+    $count++;
+
+    # If we've made it down to here, we're home free; just call the
+    # post_fetch callback
+
+    next ROW unless ( $obj->_fetch_post_process( $p, $sec_level ) );
+    push @obj_list, $obj;
+  }
+  $sth->finish;
+  return \@obj_list;
+}
+
+
+# Find the number of objects that would have been returned from a
+# query, including security. Note that we only fetch the ID field
+# here...
+
+sub fetch_count {
+  my ( $class, $p ) = @_;
+  $p->{select} = [ $class->id_field() ];
+  my $sth = $class->_execute_multiple_record_query( $p );
+  my $count = 0;
+  while ( my $row = $sth->fetch ) {
+    eval { $class->check_action_security({ id       => $row->[0],
+                                           required => SEC_LEVEL_READ }) };
+    next if ( $@ );
+    $count++;
+  }
+  return $count;
+}
+
+
+sub _execute_multiple_record_query {
+  my ( $class, $p ) = @_;
+  $p->{from}   ||= [ $class->table_name ];
+  $p->{select} ||= $class->_fetch_select_fields( $p );
+  $p->{return}   = 'sth';
+  $p->{DEBUG}  ||= DEBUG_FETCH;
+
+  # We return a DBI statement handle here so we can scroll through the
+  # rows without assigning them all.
+
+  return $class->db_select( $p );
+}
+
+# field_list has precedence, then column_group, then 'all'
+
+sub _fetch_select_fields {
+  my ( $class, $p ) = @_;
+  my $field_list = $p->{field_list};
+
+  # If we were given a column group then grab its fields, ensuring
+  # that the ID field is in there as well. If the column group
+  # specified doesn't exist, then the $field_list is empty and will be
+  # filled with all the fields below.
+
+  if ( ! $field_list and $p->{column_group} ) {
+    DEBUG() && _w( 1, "Trying to retrieve fields for column group ($p->{column_group})" );
+    my $column_defs = $class->CONFIG->{column_group} || {};
+    $field_list = $column_defs->{ $p->{column_group} };
+    if ( ref $field_list eq 'ARRAY' and scalar @{ $field_list } ) {
+      my $id_field = $class->CONFIG->{id_field};
+      my %field_hash = map { $_ => 1 } @{ $field_list };
+      $field_hash{ $id_field }++;
+      $field_list = [ keys %field_hash ];
+      DEBUG() && _w( 2, "Found field list from column group: ", Dumper( $field_list ) );
     }
   }
 
+  # If the fields weren't passed in and we're not using a column
+  # group, then use all the fields
+
+  $field_list ||= $class->field_list;
+  return ( $field_list, $class->format_select( $field_list, $p->{field_alter} ) );
+}
+
+
+# Assign the fetched values to an object. Note that we clear all the
+# 'loaded' flags first.
+
+sub _fetch_assign_row {
+  my ( $self, $fields, $row ) = @_;
+  DEBUG() && _wm( 1, DEBUG_FETCH, "Setting data from row into", ref $self );
+  $self->clear_all_loaded();
+  foreach my $i ( 0 .. ( scalar @{ $row } - 1 ) ) {
+    DEBUG() && _wm( 2, DEBUG_FETCH, sprintf( " %-20s --> %s", $fields->[ $i ], substr( $row->[ $i ], 0, 10 ) ) );
+    $self->{ $fields->[ $i ] } = $row->[ $i ];
+    $self->set_loaded( $fields->[ $i ] );
+  }
+}
+
+
+sub _fetch_post_process {
+  my ( $self, $p, $level ) = @_;
+
   # Create an entry for this object in the cache unless either the
   # class or this call to fetch() doesn't want us to.
 
-  $obj->set_cached_object( $p );
+  $self->set_cached_object( $p );
 
   # Execute any actions the class (or any parent) wants after 
   # creating the object (see SPOPS.pm)
   
-  return undef unless ( $obj->post_fetch_action( $p ) );
+  return undef unless ( $self->post_fetch_action( $p ) );
   
   # Clear the 'changed' flag
   
-  $obj->clear_change;
+  $self->clear_change;
   
   # Mark this as being a saved object
 
-  $obj->has_save;
+  $self->has_save;
 
   # Set the security fetched from above into this object
   # as a temporary property (see SPOPS::Tie for more info 
   # on temporary properties); note that this is set whether
   # we retrieve a cached copy or not
-  
-  $obj->{tmp_security_level} = $level;
-  return $obj;
+
+  $self->{tmp_security_level} = $level;
+  DEBUG() && _wm( 1, DEBUG_FETCH, 
+                  ref $self, "(", $self->id, ") : cache set (if available),",
+                  "post_fetch_action() done, change flag cleared and save ",
+                  "flag set. Security: $level" );
+  return $self;
 }
 
-sub fetch_group {
-  my ( $class, $p ) = @_;
 
-  # Not sure what $p->{select} can be used for here;
-  # perhaps allow the user to pass option to return just
-  # the list of rows returned and do whatever desired with them...
+# This is to be passed to SPOPS::Tie as a coderef so it can do a
+# lazy-load a field that hasn't yet been loaded (fetched) so instead
+# of having an inner (unnamed) sub doing the work, we just create a
+# wrapper.
 
-  my @select = ( join( '.', $class->table_name, $class->id_field ) );
-  push @select, @{ $p->{select} } if ( ref $p->{select} eq 'ARRAY' );
+sub get_lazy_load_sub {
+  return \&perform_lazy_load;
+}
 
-  # Some databases have difficulty sorting by a value not
-  # specified in the SELECT clause (particularly with the 
-  # DISTINCT modifier we use), so fix that. Note that we need
-  # to strip out any modifiers (ASC, DESC, etc.) so they don't
-  # accidentally get added.
-
- if ( $p->{order} ) {
-   my $field_order = $p->{order};
-   $field_order =~ s/\bASC\b//gi;
-   $field_order =~ s/\bDESC\b//gi;
-   push @select, split /\s*,\s*/, $field_order;
- }
-
-  $p->{from} ||= [ $class->table_name ];
-  $p->{select} = \@select;
-  $p->{return} = 'list';
-  $p->{select_modifier} = 'DISTINCT';
-  my $rows = eval { $class->db_select( $p ); };
+sub perform_lazy_load {
+  my ( $class, $data, $field ) = @_;
+#  DEBUG() && _w( 2, "Performing lazy load for $class -> $field" );
+  _w( 0, "Performing lazy load for $class -> $field" );
+  die "No object data given -- cannot lazy load!\n"  unless ( ref $data eq 'HASH' );
+  die "No field given -- cannot lazy load!\n"        unless ( $field );
+  my %args = ( from   => [ $class->table_name ],
+               select => [ $field ],
+               where  => $class->id_clause( $data->{ $class->id_field } ),
+               return => 'single' );
+  my $row = eval { $class->db_select( \%args ); };
   if ( $@ ) {
-    $SPOPS::Error::user_msg = 'Error retrieving records from database';
+    $SPOPS::Error::user_msg = "Error retrieving single field ($field) on lazy load";
     die $SPOPS::Error::user_msg;
   }
-  my @obj_list = ();
-  foreach my $row_info  ( @{ $rows } ) {
-
-    # We break this up into two statements to ensure
-    # that any 'undef's being returned (for security reasons) 
-    # don't get pushed into the list of objects
-    
-    my $obj = eval { $class->fetch( $row_info->[0], 
-                                    { db => $p->{db},
-                                      field_alter => $p->{field_alter}, 
-                                      skip_security => $p->{skip_security},
-                                      skip_cache => $p->{skip_cache} } ); };
-    if ( $@ ) {
-      _w( 0, "Cannot fetch $row_info->[0] from $class. Error: $@" );
-    }
-    push @obj_list, $obj  if ( $obj );
-  }
-  return \@obj_list;
+  return $row->[0];
 }
 
+
 sub save {
   my ( $self, $p ) = @_; 
   my $DEBUG = DEBUG_SAVE || $p->{DEBUG};
-  _wm( 1, $DEBUG, "Trying to save a <<", ref $self, ">>" );
+  $DEBUG && _wm( 1, $DEBUG, "Trying to save a <<", ref $self, ">>" );
   my $id = $self->id;
 
   # We can force save() to be an INSERT by passing in a true value for
@@ -455,7 +597,7 @@ sub save {
   # anything.
 
   unless ( $is_add or $self->changed ) {
-    _wm( 1, $DEBUG, "This object exists and has not changed. Exiting." );
+    $DEBUG && _wm( 1, $DEBUG, "This object exists and has not changed. Exiting." );
     return $id;
   }
 
@@ -471,7 +613,7 @@ sub save {
     $level = $self->check_action_security( { required => SEC_LEVEL_WRITE,
                                              is_add   => $is_add } );
   }
-  _wm( 1, $DEBUG, "Security check passed ok. Continuing." );
+  $DEBUG && _wm( 1, $DEBUG, "Security check passed ok. Continuing." );
 
   # Callback for objects to do something before they're saved
 
@@ -516,6 +658,10 @@ FIELD:
   unless ( $p->{skip_log} ) {
     $self->log_action( $action, $self->id );
   }
+
+  # Yes, this is now a saved object
+
+  $self->has_save;
   
   # Being newly-created/updated, there are no changes
 
@@ -535,7 +681,7 @@ sub _save_insert {
   my ( $self, $p ) = @_;
   $p ||= {};
   my $DEBUG = DEBUG_SAVE || $p->{DEBUG};
-  _wm( 1, $DEBUG, "Treating the save as an INSERT." );
+  $DEBUG && _wm( 1, $DEBUG, "Treating the save as an INSERT." );
 
   my $db = $p->{db} || $self->global_db_handle;
   
@@ -548,7 +694,7 @@ sub _save_insert {
     $self->id( $pre_id );
     push @{ $p->{field} }, $self->id_field;
     push @{ $p->{value} }, $pre_id;
-    _wm( 1, $DEBUG, "Retrieved ID before insert: $pre_id" );
+    $DEBUG && _wm( 1, $DEBUG, "Retrieved ID before insert: $pre_id" );
   }
 
   # Do the insert; ask DB to return the statement handle
@@ -582,7 +728,7 @@ sub _save_insert {
   my $post_id = $self->post_fetch_id( { %{ $p }, db => $db, statement => $sth } );
   if ( $post_id ) {	 
     $self->id( $post_id );
-    _wm( 1, $DEBUG, "ID fetched after insert: $post_id" );
+    $DEBUG && _wm( 1, $DEBUG, "ID fetched after insert: $post_id" );
   }
 
   # Here we actually re-fetch any new information from the database
@@ -595,7 +741,7 @@ sub _save_insert {
   unless ( $p->{no_sync} or $self->no_save_sync ) {
     my $fill_in_fields = $self->CONFIG->{sql_defaults} || [];
     if ( scalar @{ $fill_in_fields } ) {
-      _wm( 1, $DEBUG, "Fetching defaults for fields ", 
+      $DEBUG && _wm( 1, $DEBUG, "Fetching defaults for fields ", 
                      join( ' // ', @{ $fill_in_fields } ), " after insert." );
       my $row = eval { $self->db_select({ 
                                  from   => [ $self->table_name ],
@@ -603,17 +749,18 @@ sub _save_insert {
                                  where  => $self->id_clause( undef, undef, $p ),
                                  db     => $p->{db},
                                  return => 'single' }) };
-     
-      # Even though there was an error, we probably (?)
-      # want to continue processing... I'm ambivalent about this,
-      # however...
+
+      # Even though there was an error, we probably want to continue
+      # processing... I'm ambivalent about this.
 
       if ( $@ ) {
         $SPOPS::Error::user_msg = 'Cannot re-fetch row. Continuing with normal process';
+        _w( 0, $SPOPS::Error::user_msg, "(Simple: $@)", 
+              "(Enhanced: $SPOPS::Error::system_msg)" );
       }
       else {
         for ( my $i = 0; $i < scalar @{ $fill_in_fields }; $i++ ) {
-          _wm( 1, $DEBUG, "Setting $fill_in_fields->[$i] to $row->[$i]" );
+          $DEBUG && _wm( 2, $DEBUG, "Setting $fill_in_fields->[$i] to $row->[$i]" );
           $self->{ $fill_in_fields->[ $i ] } = $row->[ $i ];
         }
       }
@@ -624,8 +771,8 @@ sub _save_insert {
   # we have requested to skip it
   
   unless ( $p->{skip_security} ) {
-    eval { $self->create_initial_security( { oid => $self->id } ); };
-    warn " (DBI/save): Error creating initial security: $@"  if ( $@ );
+    eval { $self->create_initial_security({ object_id => $self->id }) };
+    _w( 0, "Error creating initial security: $@" ) if ( $@ );
   }
   return 1; 
 }
@@ -641,14 +788,14 @@ sub _save_update {
   my $id_clause = ( $p->{use_id} ) 
                      ? $self->id_clause( $p->{use_id}, undef, $p ) 
                      : $self->id_clause( undef, undef, $p );
- _wm( 1, $DEBUG, "Processing save as UPDATE with clause ($id_clause)" );
+  $DEBUG && _wm( 1, $DEBUG, "Processing save as UPDATE with clause ($id_clause)" );
 
- # Note that the 'field' and 'value' parameters are in $p and 
- # exist when the hashref is expanded into %args
+  # Note that the 'field' and 'value' parameters are in $p and 
+  # exist when the hashref is expanded into %args
 
- my %args = ( where => $id_clause, 
-              table => $self->table_name, 
-              %{ $p } );
+  my %args = ( where => $id_clause, 
+               table => $self->table_name, 
+               %{ $p } );
   my $rv =  eval { $self->db_update( \%args ); };
   if ( $@ ) {
     _w( 0, "Update failed! Args: ", Dumper( \%args ), "$SPOPS::Error::system_msg" );
@@ -665,15 +812,16 @@ sub remove {
   my ( $self, $p ) = @_;
 
   # Don't remove it unless it's been saved already
+
   return undef   unless ( $self->has_save );
   my $DEBUG = DEBUG || $p->{DEBUG};
 
   my $level = SEC_LEVEL_WRITE;
   unless ( $p->{skip_security} ) {
-    $level = $self->check_action_security({ 
-                               required => SEC_LEVEL_WRITE });
+    $level = $self->check_action_security({ required => SEC_LEVEL_WRITE });
   }
- _wm( 1, $DEBUG, "Security check passed ok. Continuing." );
+
+  $DEBUG && _wm( 1, $DEBUG, "Security check passed ok. Continuing." );
 
   # Allow members to perform an action before getting removed
 
@@ -710,7 +858,8 @@ sub remove {
   return undef unless ( $self->post_remove_action( $p ) );
  
   # ... and log the deletion
-  $self->log_action( 'delete', $id );
+
+  $self->log_action( 'delete', $id ) unless ( $p->{skip_log} );
   return 1;
 }
 
@@ -788,50 +937,96 @@ Create the database table the class depends on.
 
 Complete!
 
+=head1 DATABASE HANDLE
+
+This package requires access to a database handle. We do not keep a
+reference of the database handle with the object for complexity
+reasons (but you can actually implement this if you would
+like). Instead you either need to pass a database handle to a method
+using the C<db> parameter or define a method in your object
+C<global_db_handle()> which returns an appropriate database handle.
+
+The latter sounds more difficult than it is. And if you have many
+objects using the same handle it is definitely the way to go. For
+instance, your database handle class could look like:
+
+ package My::Object::DBI;
+
+ use strict;
+
+ # These next two are optional but enable you to change databases for
+ # lots of objects very quickly.
+
+ use SPOPS::DBI::Pg;
+ @My::Object::DBI::ISA = qw( SPOPS::DBI::Pg );
+
+ use constant DBI_DSN  => 'DBI:Pg:dbname=postgres';
+ use constant DBI_USER => 'postgres';
+ use constant DBI_PASS => 'postgres';
+
+ my ( $DB );
+
+ sub global_db_handle {
+   unless ( ref $DB ) {
+     $DB = DBI->connect( DBI_DSN, DBI_USER, DBI_PASS,
+                         { RaiseError => 1, LongReadLen => 65536, LongTruncOk => 0 } )
+               || die "Cannot connect! $DBI::errstr";
+   }
+   return $DB;
+ }
+
+ 1;
+
+And all your objects can use this method simply by putting the class
+in their 'isa' configuration key:
+
+ $conf = {
+    myobj => {
+       isa => [ qw/ My::Object::DBI / ],
+    },
+ };
+
+Now, your objects will have transparent access to a DBI data source.
+
 =head1 DATA ACCESS METHODS
 
-The following methods access configuration information about
-the class but are specific to the DBI subclass.
+The following methods access configuration information about the class
+but are specific to the DBI subclass. You can call all of them from
+either the class (or subclass) or an instantiated object.
 
-=over 4
+B<base_table> (Returns: $)
 
-=item *
+Just the table name, no owners or db names prepended.
 
-base_table ($): Just the table name, no owners or db names prepended.
+B<table_name> (Returns: $)
 
-=item *
+Fully-qualified table name
 
-table_name ($): Fully-qualified table name
+B<field> (Returns: \%)
 
-=item *
+Hashref of fields/properties (field is key, value is true)
 
-field (\%): Hashref of fields/properties (field is key, value is true)
+B<field_list> (Returns: \@)
 
-=item *
+Arrayref of fields/propreties
 
-field_list (\@): Arrayref of fields/propreties
+B<no_insert> (Returns: \%)
 
-=item *
+Hashref of fields not to insert (field is key, value is true)
 
-no_insert (\%): Hashref of fields not to insert (field is key, value
-is true)
+B<no_update> (Returns: \%)
 
-=item *
+Hashref of fields not to update (field is key, value is true)
 
-no_update (\%): Hashref of fields not to update (field is key, value
-is true)
+B<skip_undef> (Returns: \%)
 
-=item *
+Hashref of fields to skip update/insert if they are undefined (field
+is key, value is true)
 
-skip_undef (\%): Hashref of fields to skip update/insert if they are
-undefined (field is key, value is true)
+B<field_alter> (Returns: \%)
 
-=item *
-
-field_alter (\%): Hashref of data-formatting instructions (field is
-key, instruction is value)
-
-=back
+Hashref of data-formatting instructions (field is key, instruction is
+value)
 
 =head1 OBJECT METHODS
 
@@ -898,35 +1093,42 @@ object within the object itself. Check the temporary property
 
 Parameters:
 
- field_alter - (\%) fields are keys, values are database-dependent
-  formatting strings. You can accomplish different types of
-  date-formatting or other manipulation tricks.
+=over 4
+
+=item *
+
+B<column_group> ($) (optional)
+
+Name a group of columns you want to fetch. Only the values for these
+columns will be retrieved, and an arrayref of 
+
+B<field_alter> (\%) (optional)
+
+fields are keys, values are database-dependent formatting strings. You
+can accomplish different types of date-formatting or other
+manipulation tricks.
+
+=item *
+
+B<DEBUG> (bool) (optional)
 
 You can also pass a DEBUG value to get debugging information for that
 particular statement dumped into the error log:
 
  my $obj = eval { $class->fetch( $id, { DEBUG => 1 } ); };
 
+=back
+
 B<fetch_group( \%params )>
 
 Returns an arrayref of objects that meet the criteria you 
 specify.
 
-Parameters: 
-
- where - a WHERE clause; leave this blank and you will get all entries
-
- order - an ORDER BY clause; leave this blank and the order is
-  arbitrary
-
- other parameters get passed onto the fetch() statement when the
- records are being retrieved.
-
 This is actually fairly powerful. Examples:
 
  # Get all the user objects and put them in a hash
  # indexed by the id
- my %uid = map { $_->id => $_ } @{ $R->user->fetch_group( { order => 'last_name' } ) }; 
+ my %uid = map { $_->id => $_ } @{ $R->user->fetch_group({ order => 'last_name' }) }; 
 
  # Get only those user objects for people who have
  # logged in today
@@ -941,11 +1143,87 @@ This is actually fairly powerful. Examples:
 Note that you can also return objects that match the results of a 
 join query:
 
- my $list = eval { $class->fetch_group( { order => 'item.this, item.that',
-                                          from => [ 'item', 'modifier' ],
-                                          where => 'modifier.property = ? AND ' .
-                                                   'item.item_id = modifier.item_id',
-                                          value => [ 'property value' ], } ); };
+ my $list = eval { $class->fetch_group({ 
+                               order => 'item.this, item.that',
+                               from => [ 'item', 'modifier' ],
+                               where => 'modifier.property = ? AND ' .
+                                        'item.item_id = modifier.item_id',
+                               value => [ 'property value' ] } ) };
+
+And you can use parameters found in C<fetch()>:
+
+ my $list = eval { $class->fetch_group({ column_group => 'minimal' }) };
+
+Parameters: 
+
+=over 4
+
+=item *
+
+B<where> ($)
+
+A WHERE clause; leave this blank and you will get all entries
+
+B<value> (\@)
+
+If you use placeholders in your WHERE clause, put the values in order
+in this parameter and they will be properly quoted.
+
+B<order> ($)
+
+An ORDER BY clause; leave this blank and the order is arbitrary
+(whatever is normally returned by your database). Note that you can
+have ORDER BY clauses that use more than one field, such as:
+
+ order => 'active_date, updated_date DESC'
+
+B<limit> ($)
+
+You can limit the number of objects returned from this method. For
+example, you might run a search but allow the user to specify a
+maximum of 50 results per page. For each successive page displayed you
+can retrieve only those specific results.
+
+For instance, the following will return the first 10 records of a
+result set:
+
+ my $records = eval { $object_class->fetch_group({ where => "field = ?",
+                                                   value => [ 'foo' ],
+                                                   limit => '10' }) };
+
+You can also return a particular number of records offset from the
+beginning. The following will return results 21-30 of the result set.
+
+ my $records = eval { $object_class->fetch_group({ where => "field = ?",
+                                                   value => [ 'foo' ],
+                                                   limit => '20,10' }) };
+
+Other parameters get passed onto the fetch() statement when the
+records are being retrieved.
+
+=back
+
+B<fetch_count( \%params )>
+
+Returns the number of objects that would have been returned from a
+query. Note that this B<INCLUDES SECURITY CHECKS>. So running:
+
+ my $query = { where => 'field = ?',
+               value => 'broom' };
+ my $just_count = $class->fetch_count( $query );
+ $query->{order} = 'other_thingy';
+ my $rows = $class->fetch_group( $query );
+ print ( $just_count == scalar @{ $rows } )
+       ? "Equal!"
+       : "Not equal -- something's wrong!";
+
+Should print "Equal!"
+
+This method takes mostly the same parameters as C<fetch_group()>,
+although ones like 'order' will not any functionality to the query but
+will add time.
+
+Parameters not used: 'limit'
 
 B<save( [ \%params ] )>
 
@@ -1033,6 +1311,20 @@ This method is passed the action performed upon an object ('create',
 'update', 'remove') and the ID. SPOPS::DBI comes with an empty method,
 but you can subclass it and do what you wish
 
+=head1 LAZY LOADING
+
+This class supports lazy loading, available in SPOPS 0.40 and
+later. All you need to do is define one or more 'column_group' entries
+in the configuration of your object and L<SPOPS> will do the rest.
+
+If you are interested: the method C<perform_lazy_load()> does the
+actual fetch of the field value.
+
+B<Important Note>: If you use lazy loading, you B<must> define a
+method C<global_db_handle()> (see L<DATABASE HANDLE> above) for your
+object -- otherwise the C<perform_lazy_load()> method will not be able
+to get it.
+
 =head1 ERROR HANDLING
 
 Like all SPOPS classes, any errors encountered will be tossed up to
@@ -1042,6 +1334,14 @@ package variables; see that module for more details.
 
 =head1 TO DO
 
+B<Consistent Field Handling>
+
+Since we use the {field_alter} directive to modify what is actually
+selected from the database and use the _fetch_assign_row() to map
+fieldnames to field positions, we need to have a generic way to map
+these two things to each other. (It is not that difficult, just making
+a note to deal with it later.)
+
 =head1 BUGS
 
 =head1 COPYRIGHT
@@ -1063,6 +1363,12 @@ The following people added to this module with patches and/or advice:
 
 Christian Lemburg <clemburg@aixonix.de>
 
+=item *
+
+Rusty Foster <rusty@kuro5hin.org> -- added the 'limit' functionality
+to fetch_group() and was also influential in the early days of this
+library.
+
 =back
 
 =cut
